#include <FastLED.h>
#include <UsiSerial.h>
#include <USI_UART_config.h>

// Runs out of memory when using the "interp" command. Determine why that command uses 1824 bytes of storacge.

// Pin assignments.
#define TX          0 // Tx pin
#define RX          1 // Rx pin
#define AMMO_PIN    2 // Reload button
#define FIRE_PIN    3 // Fire button
#define NEO_PIN     4 // NeoPixel data pin

// Values.
#define PIXEL_COUNT 5 // Number of pixels
int counter = 20;     // Full ammo count
int Red[] = {255, 0, 0};
int Yellow[] = {0, 255, 255};
int Green[] = {0, 255, 0};
int Black[] = {0, 0, 0};
int buttonState = 0; // Default button state

//SoftwareSerial ss = SoftwareSerial(TX, RX);
//Adafruit_NeoPixel strip = Adafruit_NeoPixel(PIXEL_COUNT, NEO_PIN);
CRGB leds[PIXEL_COUNT];

bool oldStateF = HIGH;  // Default state of the fire button on startup.
bool oldStateA = LOW;   // Default state of the ammo door button on startup. 
                        // ^^ If set to HIGH, could trigger reload FX at startup. ^^

void setup() {
  // initialize pins
  pinMode(AMMO_PIN, INPUT_PULLUP);
  pinMode(FIRE_PIN, INPUT_PULLUP);
  pinMode(NEO_PIN, OUTPUT);

  FastLED.addLeds<NEOPIXEL, NEO_PIN>(leds, PIXEL_COUNT);

  UsiSerial.begin();
  UsiSerial.write("#0"); // plays first track on startup POWERUP SOUND
  delay(1000); // pause for POWERUP SOUND
}

void loop() {
  // Get current button state.
  bool newStateF = digitalRead(FIRE_PIN);
  bool newStateA = digitalRead(AMMO_PIN);

  // Interpolate between red, yellow, and green to show remaining ammo.
  // This integer based approach take 762 bytes of memory.
  if (counter <= 10){
    leds[0].setRGB(-25 * counter + 255, 25 * counter, 25 * counter);
    FastLED.show();
  }
  if (counter > 10){
    leds[0].setRGB(0, 255, -25 * counter + 510);
    FastLED.show();
  }

  // Check if fire button state changed from high to low (button press) and ammo door is closed.
  if (newStateF == LOW && oldStateF == HIGH && newStateA == LOW) {
    // Short delay to debounce button.
    delay(20);
    // Check if button is still low after debounce.
    newStateF = digitalRead(FIRE_PIN);
    if (newStateF == LOW) {
      if (counter == 0) {
        // DUD PATTERN /////////////////////////////////////////////////////////////////////
        UsiSerial.write("q"); // Stop current sound file.
        delay(10);
        UsiSerial.write("#2"); // Play the FireDud sound file.
      }
      else {
        // Single pulse of color down strip
        for(int i = 1; i < PIXEL_COUNT + 1; i++) { // Start at one to skip first LED.
          leds[i].setRGB(0, 255, 0); // Activate leader color.
          if (i > 1) {
            leds[i-1].setRGB(0, 0, 0); // Turn off LED behind the leader.
          }
          ; // This sends the updated pixel color to the hardware.
          delay(20); // Delay for a period of time (in milliseconds).
          FastLED.show();
        }
        UsiSerial.write("q"); // Stop current sound file.
        delay(10);
        UsiSerial.write("#1"); // Play the FireDud sound file.
        counter = counter - 1;
      }
    }
  }
  // Check if ammo door button state changed from low to high (door opened).
  if (newStateA == HIGH && oldStateA == LOW) {
    // Short delay to debounce button.
    delay(20);
    // Check if button is still high after debounce.
    newStateA = digitalRead(AMMO_PIN);
    if (newStateA == HIGH) {
      // This section allows different sounds if there is still ammo remaining.
      if (counter == 0) {
      // ADD PATTERN FOR ReloadOpen ////////////////////////////////////////////////////////
        UsiSerial.write("q"); // Stop current sound file.
        delay(10);
        UsiSerial.write("#4"); // Play the ReloadOpen sound file.
        delay(2000);      // Wait for two seconds.
        UsiSerial.write("#6"); // Play AmmoIn sound file.
      }
      else {
      // ADD PATTERN FOR Unload ////////////////////////////////////////////////////////////
        UsiSerial.write("q"); // Stop current sound file.
        delay(10);
        UsiSerial.write("#3"); // Play the Unload sound file.
        delay(2000);      // Wait for two seconds.
        UsiSerial.write("#6"); // Play AmmoIn sound file.
      }
    }
  }
  // Check if ammo door button state changed from high to low (door closed).
  if (newStateA == HIGH && oldStateA == LOW) {
    // Short delay to debounce button.
    delay(20);
    // Check if button is still high after debounce.
    newStateA = digitalRead(AMMO_PIN);
    if (newStateA == LOW) {
      // ADD PATTERN FOR POWER UP //////////////////////////////////////////////////////////
      UsiSerial.write("q"); // Stop current sound file.
      delay(10);
      UsiSerial.write("#5"); // Play Powerup sound file
      counter = 20; // Set ammo count to 20 after the door is closed.
    }
  }
}

